@@ -1,21 +1,24 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-'''konch: Custom Python Shell
+'''konch: The customizable Python shell.
 
 Usage:
   konch
   konch init
   konch init [<config_file>] [-d]
-  konch [--file=<file>] [--shell=<shell_name>] [-d]
+  konch [--name=<name>] [-d]
+  konch [--name=<name>] [--file=<file>] [--shell=<shell_name>] [-d]
 
 Options:
   -h --help                  Show this screen.
   -v --version               Show version.
   init                       Creates a starter .konchrc file.
+  -n --name=<name>           Named config to use.
   -s --shell=<shell_name>    Shell to use. Can be either "ipy" (IPython),
                               "bpy" (BPython), or "py" (built-in Python shell),
                                or "auto" (try to use IPython or Bpython and
-                               fallback to built-in shell).
+                               fallback to built-in shell). Overrides the 'shell'
+                               option in .konchrc.
   -f --file=<file>           File path of konch config file to execute. If not provided,
                                konch will use the .konchrc file in the current
                                directory.
@@ -27,12 +30,12 @@ import logging
 import os
 import sys
 import code
-import copy
 import warnings
+import random
 
 from docopt import docopt
 
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 __author__ = 'Steven Loria'
 __license__ = 'MIT'
 
@@ -48,35 +51,30 @@ Context:
 {context}
 """
 
-DEFAULT_BANNER_TEXT = 'Welcome to the konch shell. Happy hacking!'
+DEFAULT_BANNER_TEXT = 'This is your konch shell. Happy hacking.'
 
 DEFAULT_CONFIG_FILE = '.konchrc'
 
-INIT_TEMPLATE = """#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-import os
-import random
-
+INIT_TEMPLATE = '''# -*- coding: utf-8 -*-
 import konch
 
 # TODO: Edit me
-context = {
-    'os': os,
-    'random': random,
-}
-
-# Available options: "context", "banner", "shell"
+# Available options: 'context', 'banner', 'shell'
 konch.config({
-    'context': context,
+    'context': {
+        'speak': konch.speak
+    }
 })
-"""
+'''
 
 
 def execute_file(fname, globals_=None, locals_=None):
+    """Executes code in a file. Python 2/3-compatible."""
     exec(compile(open(fname, "rb").read(), fname, 'exec'), globals_, locals_)
 
 
 def format_context(context):
+    """Output the a context dictionary as a string."""
     if context is None:
         return ''
     line_format = '{name}: {obj!r}'
@@ -87,6 +85,9 @@ def format_context(context):
 
 
 def make_banner(text=None, context=None):
+    """Generates a full banner with version info, the given text, and a
+    formatted list of context variables.
+    """
     banner_text = text or DEFAULT_BANNER_TEXT
     out = BANNER_TEMPLATE.format(version=sys.version, text=banner_text)
     if context:
@@ -94,6 +95,15 @@ def make_banner(text=None, context=None):
     return out
 
 
+def context_list2dict(context_list):
+    """Converts a list of objects (functions, classes, or modules) to a
+    dictionary mapping the object names to the objects.
+    """
+    return dict(
+        (obj.__name__, obj) for obj in context_list
+    )
+
+
 class Shell(object):
     """Base shell class.
 
@@ -181,24 +191,79 @@ SHELL_MAP = {
     'auto': AutoShell,
 }
 
+CONCHES = [
+    ('"My conch told me to come save you guys."\n'
+    '"Hooray for the magic conches!"'),
+    '"All hail the Magic Conch!"',
+    '"Hooray for the magic conches!"',
+    '"Uh, hello there. Magic Conch, I was wondering... '
+    'should I have the spaghetti or the turkey?"',
+    '"This copyrighted conch is the cornerstone of our organization."',
+    '"Praise the Magic Conch!"',
+    '"the conch exploded into a thousand white fragments and ceased to exist."',
+    '"S\'right. It\'s a shell!"',
+    '"Ralph felt a kind of affectionate reverence for the conch"',
+    '"Conch! Conch!"',
+    '"That\'s why you got the conch out of the water"',
+    '"the summons of the conch"',
+    '"Whoever holds the conch gets to speak."',
+    '"They\'ll come when they hear us--"',
+    '"We gotta drop the load!"',
+    '"Dude, we\'re falling right out the sky!!"',
+    ('"Oh, Magic Conch Shell, what do we need to do to get out of the Kelp Forest?"\n'
+        '"Nothing."'),
+    '"The shell knows all!"',
+    '"we must never question the wisdom of the Magic Conch."',
+    '"The Magic Conch! A club member!"'
+]
 
-DEFAULT_OPTIONS = {
-    'shell': AutoShell,
-    'banner': DEFAULT_BANNER_TEXT,
-    'context': {}
+
+def speak():
+    return random.choice(CONCHES)
+
+
+class Config(dict):
+    """A dict-like config object. Behaves like a normal dict except that
+    the ``context`` will always be converted from a list to a dict.
+    """
+
+    def __init__(self, context=None, banner=None, shell=AutoShell):
+        ctx = Config.transform_val(context) or {}
+        super(Config, self).__init__(context=ctx, banner=banner, shell=shell)
+
+    def __setitem__(self, key, value):
+        val = Config.transform_val(value)
+        super(Config, self).__setitem__(key, val)
+
+    @staticmethod
+    def transform_val(val):
+        if isinstance(val, (list, tuple)):
+            return context_list2dict(val)
+        return val
+
+    def update(self, d):
+        for key in d.keys():
+            self[key] = d[key]
+
+# cfg and config_registry are global variables that may be mutated by a
+# .konchrc file
+cfg = Config()
+config_registry = {
+    'default': cfg
 }
 
-#: Global configuration object. Defines default options for start().
-cfg = copy.deepcopy(DEFAULT_OPTIONS)
-
 
 def start(context, banner=None, shell=AutoShell):
     """Start up the konch shell with a given context."""
+    logger.debug('Using shell...')
+    logger.debug(shell)
+    if banner is None:
+        banner = speak()
     shell(context, banner).start()
 
 
 def config(config_dict):
-    """Configures the konch shell. This function should be called in your
+    """Configures the konch shell. This function should be called in a
     .konchrc file.
 
     :param dict config_dict: Dict that may contain 'context', 'banner', and/or
@@ -209,24 +274,41 @@ def config(config_dict):
     return cfg
 
 
+def named_config(name, config_dict):
+    """Adds a named config to the config registry.
+    This function should be called in a .konchrc file.
+    """
+    global config_registry
+    config_registry[name] = Config(**config_dict)
+
+
 def reset_config():
     global cfg
-    cfg = copy.deepcopy(DEFAULT_OPTIONS)
+    cfg = Config()
     return cfg
 
 
-def __update_cfg_from_args(args):
+def get_file_directory(filename):
+    return os.path.dirname(os.path.abspath(filename))
+
+
+def __ensure_directory_in_path(filename):
+    directory = get_file_directory(filename)
+    if directory not in sys.path:
+        logger.debug('Adding {0} to sys.path'.format(directory))
+        sys.path.insert(0, directory)
+
+
+def use_file(filename):
     # First update cfg by executing the config file
-    config_file = args['--file'] or DEFAULT_CONFIG_FILE
+    config_file = filename or DEFAULT_CONFIG_FILE
     if os.path.exists(config_file):
         logger.info('Using {0}'.format(config_file))
+        # Ensure that relative imports are possible
+        __ensure_directory_in_path(config_file)
         execute_file(config_file)
     else:
         warnings.warn('"{0}" not found.'.format(config_file))
-    # Allow default shell to be overriden by command-line argument
-    shell_name = args['--shell']
-    if shell_name:
-        cfg['shell'] = SHELL_MAP.get(shell_name.lower(), AutoShell)
     return cfg
 
 
@@ -257,8 +339,19 @@ def main():
     if args['init']:
         config_file = args['<config_file>'] or DEFAULT_CONFIG_FILE
         init_config(config_file)
-    __update_cfg_from_args(args)
-    start(**cfg)
+    use_file(args['--file'])
+
+    if args['--name']:
+        config = config_registry.get(args['--name'], cfg)
+        logger.debug('Using named config...')
+        logger.debug(config)
+    else:
+        config = cfg
+    # Allow default shell to be overriden by command-line argument
+    shell_name = args['--shell']
+    if shell_name:
+        config['shell'] = SHELL_MAP.get(shell_name.lower(), AutoShell)
+    start(**config)
     sys.exit(0)
 
 if __name__ == '__main__':
